@@ -1,4 +1,4 @@
-# $Id: ODBC.pm 15448 2012-10-23 08:35:03Z mjevans $
+# $Id: ODBC.pm 15565 2013-01-25 09:44:03Z mjevans $
 #
 # Copyright (c) 1994,1995,1996,1998  Tim Bunce
 # portions Copyright (c) 1997-2004  Jeff Urlwin
@@ -19,7 +19,7 @@ require 5.008;
 # see discussion on dbi-users at
 # http://www.nntp.perl.org/group/perl.dbi.dev/2010/07/msg6096.html and
 # http://www.dagolden.com/index.php/369/version-numbers-should-be-boring/
-$DBD::ODBC::VERSION = '1.41';
+$DBD::ODBC::VERSION = '1.43';
 
 {
     ## no critic (ProhibitMagicNumbers ProhibitExplicitISA)
@@ -32,7 +32,7 @@ $DBD::ODBC::VERSION = '1.41';
 
     @ISA = qw(Exporter DynaLoader);
 
-    # my $Revision = substr(q$Id: ODBC.pm 15448 2012-10-23 08:35:03Z mjevans $, 13,2);
+    # my $Revision = substr(q$Id: ODBC.pm 15565 2013-01-25 09:44:03Z mjevans $, 13,2);
 
     require_version DBI 1.609;
 
@@ -112,6 +112,7 @@ $DBD::ODBC::VERSION = '1.41';
             'Attribution' => 'DBD::ODBC by Jeff Urlwin, Tim Bunce and Martin J. Evans',
 	    });
         DBD::ODBC::st->install_method("odbc_lob_read");
+        DBD::ODBC::st->install_method("odbc_rows", { O=>0x00000000 });
         # don't clear errors - IMA_KEEP_ERR = 0x00000004
         DBD::ODBC::st->install_method("odbc_getdiagrec", { O=>0x00000004 });
         DBD::ODBC::db->install_method("odbc_getdiagrec", { O=>0x00000004 });
@@ -194,9 +195,11 @@ $DBD::ODBC::VERSION = '1.41';
             odbc_column_display_size       => undef, # sth and dbh
             odbc_utf8_on                   => undef, # sth and dbh
             odbc_driver_complete           => undef,
-            odbc_batch_size                      => undef,
+            odbc_batch_size                => undef,
             odbc_array_operations          => undef, # sth and dbh
-            odbc_taf_callback                    => undef
+            odbc_taf_callback              => undef,
+            odbc_trace                          => undef, # dbh
+            odbc_trace_file                          => undef, # dbh
         };
     }
 
@@ -650,7 +653,7 @@ DBD::ODBC - ODBC Driver for DBI
 
 =head1 VERSION
 
-This documentation refers to DBD::ODBC version 1.41.
+This documentation refers to DBD::ODBC version 1.43.
 
 =head1 SYNOPSIS
 
@@ -1373,6 +1376,35 @@ see others I cannot create a generic interface. I'll happily accept
 patches for any other driver or if you send me a working copy of the
 driver and the documentation I will add support for it.
 
+=head3 odbc_trace_file
+
+Specify the name and path to a file you want ODBC API trace
+information to be written to. See L</odbc_trace>.
+
+=head3 odbc_trace
+
+Enable or disable ODBC API tracing. Set to 1 to enable and 0 to
+disable.
+
+This calls SQLSetConnectAttr for SQL_ATTR_TRACE and either sets
+SQL_OPT_TRACE_ON or SQL_OPT_TRACE_OFF. Enabling tracing will tell
+the ODBC driver manager to write and ODBC API trace to the file
+named with L</odbc_trace_file>.
+
+NOTE: If you don't set odbc_trace_file most ODBC Driver Managers write
+to a file called SQL.LOG in the root directory (but this depends on
+the driver manager used).
+
+NOTE: This tracing is produced by the ODBC Driver Manager and has
+nothing to do with DBD::ODBC other than it should trace the ODBC
+calls DBD::ODBC makes i.e., DBD::ODBC is not responsible for the
+tracing mechanism itself.
+
+NOTE: Enabling tracing will probably slow your application down a lot.
+I'd definitely think twice about it if in a production environment
+unless you are desperate as it tends to produce very large trace
+files for short periods of ODBC activity.
+
 =head2 Private statement attributes
 
 =head3 odbc_more_results
@@ -1393,6 +1425,30 @@ result sets have been retrieved.
 
 =head2 Private statement methods
 
+=head3 odbc_rows
+
+This method was added in 1.42_1.
+
+In 64 bit ODBC SQLRowCount can return a 64bit value for the number of
+rows affected. Unfortunately, the DBI DBD interface currently (at
+least until 1.622) defines execute as returning an int so values which
+cannot fit in an int are truncated. See RT 81911.
+
+From DBD::ODBC 1.42_1 DBD::ODBC
+
+o defines this method which will return the affected rows in an IV
+(and IVs are guaranteed to be able to hold a pointer) so you
+can get the real affected rows without truncation.
+
+o if it detects an overflow in the execute method it will issue
+a warning (if Warn is on which it is by default) and return INT_MAX.
+
+At some stage DBI may change to fix the issue this works around.
+
+NOTE: the return from odbc_rows is not the raw value returned by
+SQLRowCount. It is the same as execute normally returns e.g., 0E0 (for
+0), -1 for unknown and N for N rows affected where N > 0.
+
 =head3 odbc_lob_read
 
   $chrs_or_bytes_read = $sth->lob_read($column_no, \$lob, $length, \%attr);
@@ -1956,7 +2012,7 @@ provided at the start of type_info_all.
 
 =head3 Binding Columns
 
-The DBI specification allows a column type to be overriden in the call
+The DBI specification allows a column type to be overridden in the call
 to the bind_col method. Mostly, DBD::ODBC ignores this type as it
 binds integers (SQL_INTEGER) as a SQL_C_LONG (since DBD::ODBC 1.38_1)
 and all other columns as SQL_C_CHAR or SQL_C_WCHAR and it is too late
