@@ -1,4 +1,4 @@
-# $Id: enable_odbc_tracing.pl 15555 2013-01-17 15:54:03Z mjevans $
+# $Id: enable_odbc_tracing.pl 15562 2013-01-21 12:07:49Z mjevans $
 # Shows how to enable ODBC API tracing for this Perl script.
 # NOTE: the ODBC Driver manager does the actual tracing
 use strict;
@@ -6,10 +6,15 @@ use warnings;
 use DBI;
 
 my $h = DBI->connect($ENV{DBI_DSN}, $ENV{DBI_USER}, $ENV{DBI_PASS},
-                     {odbc_trace_file => '/tmp/odbc.trc',
+                     {odbc_trace_file => 'c:\users\martin\odbc.trc',
                       odbc_trace => 1});
+print "trace is ", $h->{odbc_trace}, ", ", $h->{odbc_trace_file}, "\n";
 my $s = $h->prepare('select 1');
 $s->execute;
+$h->{odbc_trace} = 0;
+print "trace is ", $h->{odbc_trace}, "\n";
 $s->fetch;
 $s->fetch;
+$h->{odbc_trace} = 1;
+print "trace is ", $h->{odbc_trace}, "\n";
 $h->disconnect;
